@@ -84,8 +84,13 @@ cigar_ quality_trim(bam1_t* r, uint8_t qual_threshold, uint8_t sliding_window){
   int del_len, cig, temp;
   uint32_t i = 0, j = 0;
   cigar_ t;
-  while(m >= qual_threshold && (i < (uint32_t)r->core.l_qseq)){
+  init_cigar(&t);
+  if(0 > r->core.l_qseq - sliding_window)
+    sliding_window = (uint32_t)r->core.l_qseq;
+  while(i < (uint32_t)r->core.l_qseq){
     m = mean_quality(qual, i, i+sliding_window);
+    if(m < qual_threshold)
+      break;
     i++;
     if(i > (uint32_t)r->core.l_qseq - sliding_window)
       sliding_window--;
@@ -97,7 +102,9 @@ cigar_ quality_trim(bam1_t* r, uint8_t qual_threshold, uint8_t sliding_window){
   del_len = r->core.l_qseq - i;
   start_pos = get_pos_on_reference(cigar, r->core.n_cigar, del_len, r->core.pos); // For reverse reads need to set core->pos.
   if(reverse && start_pos <= r->core.pos) {
+    free(ncigar);
     t.cigar = cigar;
+    t.free_cig = false;
     t.nlength = r->core.n_cigar;
     t.start_pos = r->core.pos;
     return t;
@@ -140,6 +147,7 @@ cigar_ quality_trim(bam1_t* r, uint8_t qual_threshold, uint8_t sliding_window){
   }
   t.cigar = ncigar;
   t.nlength = j;
+  t.free_cig = true;
   t.start_pos = start_pos;
   return t;
 }
@@ -152,21 +160,21 @@ void print_cigar(uint32_t *cigar, int nlength){
   std::cout << std::endl;
 }
 
-cigar_ primer_trim(bam1_t *r, int32_t new_pos, bool unpaired_rev = false){
+cigar_ primer_trim(bam1_t *r, bool &isize_flag, int32_t new_pos, bool unpaired_rev = false){
   uint32_t *ncigar = (uint32_t*) malloc(sizeof(uint32_t) * (r->core.n_cigar + 1)), // Maximum edit is one more element with soft mask
     *cigar = bam_get_cigar(r);
   uint32_t i = 0, j = 0;
   int max_del_len = 0, cig, temp, del_len = 0;
   bool reverse = false;
-  if((r->core.flag&BAM_FPAIRED) != 0){ // If paired
-    if (bam_is_rev(r)){
+  if((r->core.flag&BAM_FPAIRED) != 0 && isize_flag){ // If paired and isize > read length
+    if (bam_is_rev(r)){ // If -ve strand (?)
       max_del_len = bam_cigar2qlen(r->core.n_cigar, bam_get_cigar(r)) - get_pos_on_query(cigar, r->core.n_cigar, new_pos, r->core.pos) - 1;
       reverse_cigar(cigar, r->core.n_cigar);
       reverse = true;
     } else {
       max_del_len = get_pos_on_query(cigar, r->core.n_cigar, new_pos, r->core.pos);
     }
-  } else {			// Unpaired
+  } else {			// trim without considering pairing
     if(unpaired_rev){
       max_del_len = bam_cigar2qlen(r->core.n_cigar, bam_get_cigar(r)) - get_pos_on_query(cigar, r->core.n_cigar, new_pos, r->core.pos) - 1;
       reverse_cigar(cigar, r->core.n_cigar);
@@ -198,7 +206,7 @@ cigar_ primer_trim(bam1_t *r, int32_t new_pos, bool unpaired_rev = false){
 	ncigar[j] = bam_cigar_gen(n, BAM_CSOFT_CLIP);
       } else if (del_len < n && del_len > 0){
 	ncigar[j] = bam_cigar_gen(del_len, BAM_CSOFT_CLIP);
-      } else if (del_len == 0) {	// Ading insertions before start position of read
+      } else if (del_len == 0) {	// Adding insertions before start position of read
 	ncigar[j] = bam_cigar_gen(n, BAM_CSOFT_CLIP);
 	j++;
 	i++;
@@ -227,8 +235,9 @@ cigar_ primer_trim(bam1_t *r, int32_t new_pos, bool unpaired_rev = false){
   }
   return {
     ncigar,
-      j,
-      start_pos
+    true,
+    j,
+    start_pos
   };
 }
 
@@ -259,7 +268,7 @@ void get_overlapping_primers(bam1_t* r, std::vector<primer> primers, std::vector
     start_pos = r->core.pos;
   }
   for(std::vector<primer>::iterator it = primers.begin(); it != primers.end(); ++it) {
-    if(start_pos >= it->get_start() && start_pos <= it->get_end() && strand == it->get_strand())
+    if(start_pos >= it->get_start() && start_pos <= it->get_end() && (strand == it->get_strand() || it->get_strand() == 0))
       overlapped_primers.push_back(*it);
   }
 }
@@ -276,31 +285,27 @@ void get_overlapping_primers(bam1_t* r, std::vector<primer> primers, std::vector
     start_pos = r->core.pos;
   }
   for(std::vector<primer>::iterator it = primers.begin(); it != primers.end(); ++it) {
-    if(start_pos >= it->get_start() && start_pos <= it->get_end() && strand == it->get_strand())
+    if(start_pos >= it->get_start() && start_pos <= it->get_end() && (strand == it->get_strand() ||it->get_strand() == 0))
       overlapped_primers.push_back(*it);
   }
 }
 
-cigar_ condense_cigar(uint32_t* cigar, uint32_t n){
+void condense_cigar(cigar_ *t){
   uint32_t i = 0, len = 0, cig, next_cig;
-  cigar_ t;
-  while(i< n -1){
-    cig = bam_cigar_op(cigar[i]);
-    next_cig = bam_cigar_op(cigar[i+1]);
+  while(i< t->nlength -1){
+    cig = bam_cigar_op(t->cigar[i]);
+    next_cig = bam_cigar_op(t->cigar[i+1]);
     if(cig == next_cig){
-      len = bam_cigar_oplen(cigar[i])+bam_cigar_oplen(cigar[i+1]);
-      cigar[i] = bam_cigar_gen(len, bam_cigar_op(cigar[i]));
-      for(uint32_t j = i+1; j < n - 1; j++){
-	cigar[j] = cigar[j+1];
+      len = bam_cigar_oplen(t->cigar[i])+bam_cigar_oplen(t->cigar[i+1]);
+      t->cigar[i] = bam_cigar_gen(len, bam_cigar_op(t->cigar[i]));
+      for(uint32_t j = i+1; j < t->nlength - 1; j++){
+	t->cigar[j] = t->cigar[j+1];
       }
-      n--;
+      t->nlength--;
     } else {
       i++;
     }
   }
-  t.cigar = cigar;
-  t.nlength = n;
-  return t;
 }
 
 void add_pg_line_to_header(bam_hdr_t** hdr, char *cmd){
@@ -315,16 +320,53 @@ void add_pg_line_to_header(bam_hdr_t** hdr, char *cmd){
   (*hdr)->l_text = len-1;
 }
 
-int trim_bam_qual_primer(std::string bam, std::string bed, std::string bam_out, std::string region_, uint8_t min_qual, uint8_t sliding_window, std::string cmd, bool write_no_primer_reads, int min_length = 30){
-  std::vector<primer> primers;
-  if(!bed.empty()){
-    primers = populate_from_file(bed);
+// get the length of the longest primer
+int get_bigger_primer(std::vector<primer> primers){
+  int max_primer_len = 0;
+  for (auto & p : primers) {
+    if(max_primer_len < p.get_length()){
+      max_primer_len = p.get_length();
+    }
   }
-  // if(primers.size() == 0){
-  //   return 0;
-  // }
+  return max_primer_len;
+}
+
+// check if read is enveloped by any of the amplicons
+bool amplicon_filter(IntervalTree amplicons, bam1_t* r){
+  Interval fragment_coords = Interval(0, 1);
+  if(r->core.isize > 0){
+    fragment_coords.low = r->core.pos;
+    fragment_coords.high = r->core.pos + r->core.isize;
+  } else {
+    fragment_coords.low = bam_endpos(r) + r->core.isize;
+    fragment_coords.high = bam_endpos(r);
+  }
+  // debugging
+  bool amplicon_flag = amplicons.envelopSearch(fragment_coords);
+  return amplicon_flag;
+}
+
+int trim_bam_qual_primer(std::string bam, std::string bed, std::string bam_out, std::string region_, uint8_t min_qual, uint8_t sliding_window, std::string cmd, bool write_no_primer_reads, bool keep_for_reanalysis, int min_length = 30, std::string pair_info = "", int32_t primer_offset = 0) {
+  int retval = 0;
+  std::vector<primer> primers;
+  int max_primer_len = 0;
+  if(!bed.empty()){
+    primers = populate_from_file(bed, primer_offset);
+    if(primers.size() == 0){
+      std::cout << "Exiting." << std::endl;
+      return -1;
+    }
+  }
+  max_primer_len = get_bigger_primer(primers);
+  // get coordinates of each amplicon
+  IntervalTree amplicons;
+  if(!pair_info.empty()){
+    amplicons = populate_amplicons(pair_info, primers);
+  }
+  std::cout << "Amplicons detected: " << std::endl;
+  amplicons.inOrder();
   if(bam.empty()){
-    std::cout << "Bam file in empty." << std::endl;
+    std::cout << "Bam file is empty." << std::endl;
     return -1;
   }
   bam_out += ".bam";
@@ -399,48 +441,74 @@ int trim_bam_qual_primer(std::string bam, std::string bed, std::string bam_out,
   bam1_t *aln = bam_init1();
   int ctr = 0;
   cigar_ t;
+  init_cigar(&t);
   uint32_t primer_trim_count = 0, no_primer_counter = 0, low_quality = 0;
   bool unmapped_flag = false;
+  bool amplicon_flag = false;
+  bool isize_flag = true;
+  uint32_t failed_frag_size = 0;
   uint32_t unmapped_counter = 0;
+  uint32_t amplicon_flag_ctr = 0;
   primer cand_primer;
   std::vector<primer> overlapping_primers;
   std::vector<primer>::iterator cit;
   bool primer_trimmed = false;
+  //Iterate through reads
   while(sam_itr_next(in, iter, aln) >= 0) {
     unmapped_flag = false;
     primer_trimmed = false;
-    if((aln->core.flag&BAM_FUNMAP) == 0){
-      if((aln->core.flag&BAM_FPAIRED) != 0){ // If paired
+    get_overlapping_primers(aln, primers, overlapping_primers);
+    if((aln->core.flag&BAM_FUNMAP) == 0){ // If mapped
+      // if primer pair info provided, check if read correctly overlaps with atleast one amplicon
+      if(!pair_info.empty()){
+        amplicon_flag = amplicon_filter(amplicons, aln);
+        if(!amplicon_flag){
+	  if (keep_for_reanalysis) {   // -k (keep) option
+	    aln->core.flag |= BAM_FQCFAIL;
+	    if (bam_write1(out, aln) < 0) { retval = -1; goto error; }
+          }
+          amplicon_flag_ctr++;
+          continue;
+	}
+      }
+      isize_flag = (abs(aln->core.isize) - max_primer_len) > abs(aln->core.l_qseq);
+      // if reverse strand
+      if((aln->core.flag&BAM_FPAIRED) != 0 && isize_flag){ // If paired
 	get_overlapping_primers(aln, primers, overlapping_primers);
-	if(overlapping_primers.size() > 0){
+	if(overlapping_primers.size() > 0){ // If read starts before overlapping regions (?)
 	  primer_trimmed = true;
-	  if(bam_is_rev(aln)){	// Reverse
-	    cand_primer = get_min_start(overlapping_primers);
-	    t = primer_trim(aln, cand_primer.get_start() - 1, false);
-	  } else {		// Forward
-	    cand_primer = get_max_end(overlapping_primers);
-	    t = primer_trim(aln, cand_primer.get_end() + 1, false);
+	  if(bam_is_rev(aln)){	// Reverse read
+	    cand_primer = get_min_start(overlapping_primers); // fetch reverse primer (?)
+	    t = primer_trim(aln, isize_flag, cand_primer.get_start() - 1, false);
+	  } else {		// Forward read
+	    cand_primer = get_max_end(overlapping_primers); // fetch forward primer (?)
+	    t = primer_trim(aln, isize_flag, cand_primer.get_end() + 1, false);
 	    aln->core.pos += t.start_pos;
 	  }
 	  replace_cigar(aln, t.nlength, t.cigar);
+	  free(t.cigar);
 	  // Add count to primer
 	  cit = std::find(primers.begin(), primers.end(), cand_primer);
 	  if(cit != primers.end())
 	    cit->add_read_count(1);
 	}
 	t = quality_trim(aln, min_qual, sliding_window);	// Quality Trimming
-	if(bam_is_rev(aln))
+	if(bam_is_rev(aln))  // if reverse strand
 	  aln->core.pos = t.start_pos;
-	t = condense_cigar(t.cigar, t.nlength);
+	condense_cigar(&t);
 	// aln->core.pos += t.start_pos;
 	replace_cigar(aln, t.nlength, t.cigar);
       } else {			// Unpaired reads: Might be stitched reads
+        if(abs(aln->core.isize) <= abs(aln->core.l_qseq)){
+          failed_frag_size++;
+        }
 	// Forward primer
 	get_overlapping_primers(aln, primers, overlapping_primers, false);
 	if(overlapping_primers.size() > 0){
 	  primer_trimmed = true;
 	  cand_primer = get_max_end(overlapping_primers);
-	  t = primer_trim(aln, cand_primer.get_end() + 1, false);
+	  t = primer_trim(aln, isize_flag, cand_primer.get_end() + 1, false);
+	  // Update read's left-most coordinate
 	  aln->core.pos += t.start_pos;
 	  replace_cigar(aln, t.nlength, t.cigar);
 	  // Add count to primer
@@ -453,7 +521,7 @@ int trim_bam_qual_primer(std::string bam, std::string bed, std::string bam_out,
 	if(overlapping_primers.size() > 0){
 	  primer_trimmed = true;
 	  cand_primer = get_min_start(overlapping_primers);
-	  t = primer_trim(aln, cand_primer.get_start() - 1, true);
+	  t = primer_trim(aln, isize_flag, cand_primer.get_start() - 1, true);
 	  replace_cigar(aln, t.nlength, t.cigar);
 	  // Add count to primer
 	  cit = std::find(primers.begin(), primers.end(), cand_primer);
@@ -461,7 +529,9 @@ int trim_bam_qual_primer(std::string bam, std::string bed, std::string bam_out,
 	    cit->add_read_count(1);
 	}
 	t = quality_trim(aln, min_qual, sliding_window);	// Quality Trimming
-	t = condense_cigar(t.cigar, t.nlength);
+	if(bam_is_rev(aln))  // if reverse strand
+	  aln->core.pos = t.start_pos;
+	condense_cigar(&t);
 	replace_cigar(aln, t.nlength, t.cigar);
       }
       if(primer_trimmed){
@@ -476,33 +546,26 @@ int trim_bam_qual_primer(std::string bam, std::string bed, std::string bam_out,
       if(primer_trimmed){	// Write to BAM only if primer found.
 	int16_t cand_ind = cand_primer.get_indice();
 	bam_aux_append(aln, "XA", 's', sizeof(cand_ind), (uint8_t*) &cand_ind);
-	if(bam_write1(out, aln) < 0){
-	  std::cout << "Not able to write to BAM" << std::endl;
-	  hts_itr_destroy(iter);
-	  hts_idx_destroy(idx);
-	  bam_destroy1(aln);
-	  bam_hdr_destroy(header);
-	  sam_close(in);
-	  bgzf_close(out);
-	  return -1;
-	}
-      } else {
-	if((primers.size() == 0 || write_no_primer_reads) && !unmapped_flag){ // Write mapped reads to BAM if -e flag given
-	  if(bam_write1(out, aln) < 0){
-	    std::cout << "Not able to write to BAM" << std::endl;
-	    hts_itr_destroy(iter);
-	    hts_idx_destroy(idx);
-	    bam_destroy1(aln);
-	    bam_hdr_destroy(header);
-	    sam_close(in);
-	    bgzf_close(out);
-	    return -1;
-	  }
-	}
+	if(bam_write1(out, aln) < 0) { retval = -1; goto error; }
+      } else {  // no primer found
+        if (keep_for_reanalysis) {   // -k (keep) option
+          if((primers.size() == 0 || !write_no_primer_reads) && !unmapped_flag){ // -k only option
+            aln->core.flag |= BAM_FQCFAIL;
+          }
+	  if (bam_write1(out, aln) < 0) { retval = -1; goto error; }
+	} else {        // no -k option
+          if((primers.size() == 0 || write_no_primer_reads) && !unmapped_flag){ // -e only option
+	    if (bam_write1(out, aln) < 0) { retval = -1; goto error; }
+          }
+        }
 	no_primer_counter++;
       }
     } else {
       low_quality++;
+      if (keep_for_reanalysis) {
+        aln->core.flag |= BAM_FQCFAIL;
+	if (bam_write1(out, aln) < 0) { retval = -1; goto error; }
+      }
     }
     ctr++;
     if(ctr % log_skip == 0){
@@ -516,22 +579,53 @@ int trim_bam_qual_primer(std::string bam, std::string bed, std::string bam_out,
     std::cout << cit->get_name() << "\t" << cit->get_read_count() << std::endl;
   }
   std::cout << std::endl << "Trimmed primers from " << round_int(primer_trim_count, mapped) << "% (" << primer_trim_count <<  ") of reads." << std::endl;
-  std::cout << round_int( low_quality, mapped) << "% (" << low_quality << ") of reads were quality trimmed below the minimum length of " << min_length << " bp and were not writen to file." << std::endl;
+  std::cout << round_int( low_quality, mapped) << "% (" << low_quality << ") of reads were quality trimmed below the minimum length of " << min_length << " bp and were ";
+  if (keep_for_reanalysis) {
+    std::cout << "marked as failed" << std::endl;
+  } else {
+    std::cout << "not written to file." << std::endl;
+  }
   if(write_no_primer_reads){
-    std::cout << round_int(no_primer_counter, mapped) << "% ("  << no_primer_counter << ") of reads started outside of primer regions. Since the -e flag was given, these reads were written to file." << std::endl;
+    std::cout << round_int(no_primer_counter, mapped) << "% ("  << no_primer_counter << ")"
+              << " of reads started outside of primer regions. Since the "
+              << (keep_for_reanalysis ? "-ek flags were " : "-e flag was ")
+              << "given, these reads were written to file";
+    std::cout << "." << std::endl;
   } else if (primers.size() == 0) {
     std::cout << round_int(no_primer_counter, mapped) << "% ("  << no_primer_counter << ") of reads started outside of primer regions. Since there were no primers found in BED file, these reads were written to file." << std::endl;
   } else {
-    std::cout << round_int(no_primer_counter, mapped) << "% ("  << no_primer_counter << ") of reads that started outside of primer regions were not written to file." << std::endl;
+    std::cout << round_int(no_primer_counter, mapped) << "% ("  << no_primer_counter
+              << ") of reads that started outside of primer regions were ";
+    if (keep_for_reanalysis) {
+      std::cout << "written to file and marked as failed";
+    } else {
+      std::cout << "not written to file";
+    }
+    std::cout << std::endl;
   }
   if(unmapped_counter > 0){
     std::cout << unmapped_counter << " unmapped reads were not written to file." << std::endl;
   }
+  if(amplicon_flag_ctr > 0){
+    std::cout << round_int(amplicon_flag_ctr, mapped) 
+              << "% (" << amplicon_flag_ctr 
+              << ") reads were ignored because they did not fall within an amplicon" 
+              << std::endl;
+  }
+  if(failed_frag_size > 0){
+    std::cout << round_int(failed_frag_size, mapped)
+              << "% (" << failed_frag_size
+              << ") of reads had their insert size smaller than their read length"
+              << std::endl;
+  }
+
+ error:
+  if (retval) std::cout << "Not able to write to BAM" << std::endl;
   hts_itr_destroy(iter);
   hts_idx_destroy(idx);
   bam_destroy1(aln);
   bam_hdr_destroy(header);
   sam_close(in);
   bgzf_close(out);
-  return 0;
+  return retval;
 }
